With System;
With UNCHECKED_CONVERSION;
With RiA_Event_Flag_Ada83;
with ada.finalization; use ada.finalization;

Package RhpAnim Is


    Subtype Relations_Acc_T Is System.Address;
    Subtype UserClass_Acc_T Is System.Address;
    Subtype Class_Acc_T Is System.Address;
    Subtype Package_Acc_T Is System.Address;
    Subtype Task_Acc_T Is System.Address;
    Subtype State_Acc_T Is System.Address;
    Subtype Reactive_Acc_T Is System.Address;
    Subtype Event_Acc_T Is System.Address;
    Subtype String_Address Is System.Address;
    Type Event Is Null Record; 
    Type Event_T Is Access Event; 
    type Enter_Exit is new CONTROLLED with private;
    type Enter_Exit_Constructor is new CONTROLLED with private;

    Function I2A Is 
    New Unchecked_Conversion(Integer,System.Address);
    --------  Reactive class constants -----------
    Event_Not_Consumed         : Constant Integer := 0;  
    Take_Event_Error           : Constant Integer := 0;  
    Event_Consumed             : Constant Integer := 1;  
    Take_Event_Completed       : Constant Integer := 1;  
    Take_Event_In_Cleanup      : Constant Integer := 2;  
    Take_Event_Reach_Terminate : Constant Integer := 3;  

    Null_Id        : Constant Short_Integer  := - 1;  
    NoMultiplicity : Constant Integer        := - 1;  
    ZNIL           : Constant System.Address := I2A (0);  


    ------------- Init functions --------------------------------------
    --		Init - performs static initialization of the animation, 
    --		, starts animation threads, specifically the socket thread
    --
    --		Parameters:
    --			port: the animation port number.
    --			host: name of the host running rhapsody
    --					or a string containing it's IP address. 
    --            Returned value: TRUE - successful execution, otherwise-FALSE
    --------------------------------------------------------------------	
    Function Init (
          Port : Integer; 
          Host : String) 
      Return Boolean; 
 

    --------------------------------------------------------------------
    -- <?> Start - starts animation threads. </?>
    -- ELDAD - this should be removed --
    --------------------------------------------------------------------
    Procedure Start; 
    Pragma Import(C, Entity => Start,External_Name =>"anim_start");

    Procedure InitCommunicationLayer; 
    Pragma Import(C, Entity => InitCommunicationLayer,External_Name =>"RiCOSTask_InitCommunicationLayer");

    ----------------------------------------------------------
    --	Name_Global_Instance sets the user defined 
    --		name of global instance 
    --		Parameters:
    --			this: address of the instance variable
    --			Anim_Class: proxy item value (returned value of Get_Class function)
    --			InstanceName: name of the instance
    --			Multiplicity : number of instances         
    ----------------------------------------------------------
    Procedure Name_Global_Instance (
          This         :        UserClass_Acc_T; 
          Anim_Class   :        Class_Acc_T;     
          InstanceName :        String_address;          
          Multiplicity : In     Integer          ); 
    Pragma Import(C, Entity => Name_Global_Instance,External_Name =>"anim_name_global_instance");



    -------------------------------------------------------------------------
    --	Notify_Constructor - notifies Rhapsody about construction of instance
    --		returns the address, which is used by Anim_Notify_End_Constructor
    --		Parameters:
    --			this: address of the instance record 
    --			Anim_Reactive: address of Root_State_Serialize_States procedure. It is equal to ZNIL, 
    --					if class is not reactive
    --			LocalAddr: address of first local variable of constructor
    --			Anim_Class: proxy item value (returned value of Anim_Get_Class function)
    --			name: name of constructor operation
    --			signature: signature of constructor (including types of arguments) 
    --			pcount: number of arguments(it does not include the first obligatory argument) 
    --			Anim_Attr_List: address of list of arguments, created for arguments animation/trace
    -------------------------------------------------------------------------
    Procedure Notify_Constructor (
          This          : UserClass_Acc_T; 
          Anim_Reactive : Reactive_Acc_T;  
          -- It is equal to ZNIL, if class is not reactive
          EventDispatcher : UserClass_Acc_T; 
          -- The task associated with the dispatcher - usually ZNIL 
          Anim_Class     : Class_Acc_T;    
          Name,                            
          Signature      : String_Address;         
          Pcount         : Integer;        
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Notify_Constructor, External_Name =>"anim_notify_constructor");


    -------------------------------------------------------------------------
    --	Notify_Active_Constructor - notifies Rhapsody about construction
    --			 of instance of active class(task)
    --		
    --		See description of the Anim_Notify_Constructor function
    --        Events_Available_Signal - Ria_Events_Available_Signal field of active class record 
    -------------------------------------------------------------------------
    Procedure Notify_Active_Constructor (
          This          : UserClass_Acc_T; 
          Anim_Reactive : Reactive_Acc_T;  
          -- It is equal to ZNIL, if class is not reactive
           Anim_Class : Class_Acc_T;    
          Name,                        
          Signature  : String_Address;         
          Pcount     : Integer;        
          -- number of arguments
          Anim_Attr_List          : System.Address;                                         
          Events_Available_Signal : RiA_Event_Flag_Ada83.RiA_Event_Flag_Ada83_Acc_T := Null;
          Task_Name: String_Address := ZNIL); 
    Pragma Import(C, Entity => Notify_Active_Constructor,External_Name =>"anim_notify_active_constructor");


    -------------------------------------------------------------------------
    --	Notify_Destructor - notifies Rhapsody about destruction of instance
    --		Parameters:
    --			this: address of the instance record 
    --			Anim_Class: proxy item value (returned value of Anim_Get_Class function)
    --			signature: signature of destructor procedure
    -------------------------------------------------------------------------
    Procedure Notify_Destructor (
          This       : UserClass_Acc_T; 
          Anim_Class : Class_Acc_T;     
          Signature  : String_Address           ); 
    Pragma Import(C, Entity => Notify_Destructor,External_Name =>"anim_notify_destructor");


    -------------------------------------------------------------------------
    --	Notify_Operation - notifies Rhapsody about sending message 
    --		
    --		Parameters:
    --			this: address of the instance record 
    --			Anim_Class: proxy item value (returned value of Get_Class function)
    --			name: name of operation
    --			signature: signature of operation (including types of arguments) 
    --			pcount: number of arguments(it does not include the first obligatory argument) 
    --			Anim_Attr_List: address of list of arguments, created for arguments animation/trace
    -------------------------------------------------------------------------
    Procedure Notify_Operation (
          This         : UserClass_Acc_T; 
          Anim_Class   : Class_Acc_T;     
          Name,                           
          Signature    : String_Address;          
          Pcount       : Integer;         
          AttrList_Ptr : System.Address   ); 
    Pragma Import(C, Entity => Notify_Operation,External_Name =>"anim_notify_operation");

    -------------------------------------------------------------------------
    --	Notify_Function - notifies Rhapsody about global function call 
    --		
    --		Parameters:
    --			Anim_Pack: proxy item value (returned value of Anim_Get_Package function)
    --			name: name of operation
    --			signature: signature of operation (including types of arguments) 
    --			pcount: number of arguments(it does not include the first obligatory argument) 
    --			Anim_Attr_List: address of list of arguments, created for arguments animation/trace
    -------------------------------------------------------------------------
    Procedure Notify_Function (
           Anim_Pack   : Package_Acc_T;     
          Name,                           
          Signature    : String_Address;          
          Pcount       : Integer;         
          AttrList_Ptr : System.Address   ); 
    Pragma Import(C, Entity => Notify_Function,External_Name =>"anim_notify_glob_function");

    -------------------------------------------------------------------------
    --	Notify_Triggered_Operation - notifies Rhapsody about triggered operation call 
    --		
    --		See description of the Notify_Operation function
    -------------------------------------------------------------------------
    Procedure Notify_Triggered_Operation (
          This           : UserClass_Acc_T; 
          Anim_Class     : Class_Acc_T;     
          Name,                             
          Signature      : String_Address;          
          Pcount         : Integer;         
          Anim_Attr_List : System.Address   ); 
    Pragma Import(C, Entity => Notify_Triggered_Operation,External_Name =>"anim_notify_triggered_operation");


    --------------------- Serialization Attributes/Arguments ---------------------------------------
    --		Parameters
    --			attrName: name of argument or attribute
    --			attrValue: value of argument/attribute 
    --			Anim_Attr_List: address of list, containing converted to string values of arguments
    --				see similar parameter of Notify_Constructor, Notify_Active_Constructor
    --				and Notify_Operation functions
    ------------------------------------------------------------------------------------------------
 Procedure Add_Attribute (
          AttrName       : String;         
          AttrValue      : System.Address; 
          Anim_Attr_List : System.Address  ); 
    Procedure Add_Attribute (
          AttrName       : String;        
          AttrValue      : Integer;       
          Anim_Attr_List : System.Address ); 
    Procedure Add_Attribute (
          AttrName       : String;        
          AttrValue      : Float;         
          Anim_Attr_List : System.Address ); 
    Procedure Add_Attribute (
          AttrName       : String;        
          AttrValue      : Long_Float;    
          Anim_Attr_List : System.Address ); 
    Procedure Add_Attribute (
          AttrName       : String;        
          AttrValue      : Boolean;       
          Anim_Attr_List : System.Address ); 
    Procedure Add_Attribute (
          AttrName       : String;        
          AttrValue      : Duration;      
          Anim_Attr_List : System.Address ); 
    Procedure Add_Attribute (
          AttrName       : String;        
          AttrValue      : String;        
          Anim_Attr_List : System.Address ); 
    Procedure Add_Attribute (
          AttrName       : String;        
          AttrValue      : Character;     
          Anim_Attr_List : System.Address ); 
       Procedure Add_Address_Attribute (
          AttrName       : String_Address;         
          AttrValue      : System.Address; 
          Anim_Attr_List : System.Address  ); 
    Pragma Import(C, Entity => Add_Address_Attribute,External_Name =>"anim_add_address_attribute");

    Procedure Add_Integer_Attribute (
          AttrName       : String_Address;        
          AttrValue      : Integer;       
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Add_Integer_Attribute,External_Name =>"anim_add_integer_attribute");

    Procedure Add_Float_Attribute (
          AttrName       : String_Address;        
          AttrValue      : Float;         
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Add_Float_Attribute,External_Name =>"anim_add_float_attribute");

    Procedure Add_Long_Float_Attribute (
          AttrName       : String_Address;        
          AttrValue      : Long_Float;    
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Add_Long_Float_Attribute,External_Name =>"anim_add_double_attribute");

    Procedure Add_Boolean_Attribute (
          AttrName       : String_Address;        
          AttrValue      : Boolean;       
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Add_Boolean_Attribute,External_Name =>"anim_add_boolean_attribute");

    Procedure Add_Duration_Attribute (
          AttrName       : String_Address;        
          AttrValue      : Duration;      
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Add_Duration_Attribute,External_Name =>"anim_add_duration_attribute");

    Procedure Add_String_Attribute (
          AttrName       : String_Address;        
          AttrValue      : String_address;        
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Add_String_Attribute,External_Name =>"anim_add_string_attribute");

    Procedure Add_Character_Attribute (
          AttrName       : String_Address;        
          AttrValue      : String_Address;     
          Anim_Attr_List : System.Address ); 
    Pragma Import(C, Entity => Add_Character_Attribute,External_Name =>"anim_add_character_attribute");

    --------------------- Unserialization Attributes/Arguments ---------------------------------------
    --		Parameters
    --			this: class instance access 
    --                  AttrValue: value of argument/attribute 
    --			Data: List, containing all attributes 
    --			Position: address of integer, containing current offset of attribute in the list
    ------------------------------------------------------------------------------------------------
    Procedure Get_Attribute (
          This     : System.Address; 
          Data     : System.Address; 
          Position : System.Address  ); 

    Procedure Get_Attribute (
          AttrValue : In Out Integer;        
          Data      :        System.Address; 
          Position  :        System.Address  ); 

    Procedure Get_Attribute (
          AttrValue : In Out Float;          
          Data      :        System.Address; 
          Position  :        System.Address  ); 

    Procedure Get_Attribute (
          AttrValue : In Out Long_Float;     
          Data      :        System.Address; 
          Position  :        System.Address  ); 

    Procedure Get_Attribute (
          AttrValue : In Out Boolean;        
          Data      :        System.Address; 
          Position  :        System.Address  ); 

    Procedure Get_Attribute (
          AttrValue : In Out Duration;       
          Data      :        System.Address; 
          Position  :        System.Address  ); 

    Procedure Get_Attribute (
          AttrValue : In Out String;         
          Data      :        System.Address; 
          Position  :        System.Address  ); 

    Procedure Get_Attribute (
          AttrValue : In Out Character;      
          Data      :        System.Address; 
          Position  :        System.Address  ); 

    Procedure Get_Pointer_Attribute (
          This     : System.Address; 
          Data     : System.Address; 
          Position : System.Address  ); 
    Pragma Import(C, Entity => Get_Pointer_Attribute,External_Name =>"anim_get_pointer_attribute");

    Procedure Get_Integer_Attribute (
          AttrValue : System.Address; 
          Data      : System.Address; 
          Position  : System.Address  ); 
    Pragma Import(C, Entity => Get_Integer_Attribute,External_Name =>"anim_get_integer_attribute");

    Procedure Get_Long_Float_Attribute (
          AttrValue : System.Address; 
          Data      : System.Address; 
          Position  : System.Address  ); 
    Pragma Import(C, Entity => Get_Long_Float_Attribute,External_Name =>"anim_get_double_attribute");

    Procedure Get_Float_Attribute (
          AttrValue : System.Address; 
          Data      : System.Address; 
          Position  : System.Address  ); 
    Pragma Import(C, Entity => Get_Float_Attribute,External_Name =>"anim_get_float_attribute");

    Procedure Get_Boolean_Attribute (
          AttrValue : System.Address; 
          Data      : System.Address; 
          Position  : System.Address  ); 
    Pragma Import(C, Entity => Get_Boolean_Attribute,External_Name =>"anim_get_boolean_attribute");

    Procedure Get_Duration_Attribute (
          AttrValue : System.Address; 
          Data      : System.Address; 
          Position  : System.Address  ); 
    Pragma Import(C, Entity => Get_Duration_Attribute,External_Name =>"anim_get_duration_attribute");

    Procedure Get_String_Attribute (
          AttrValue : System.Address; 
          Data      : System.Address; 
          Position  : System.Address  ); 
    Pragma Import(C, Entity => Get_String_Attribute,External_Name =>"anim_get_string_attribute");

    Procedure Get_Character_Attribute (
          AttrValue : System.Address; 
          Data      : System.Address; 
          Position  : System.Address  ); 
    Pragma Import(C, Entity => Get_Character_Attribute,External_Name =>"anim_get_character_attribute");


    --------------------------------------------------------------------------------
    --            Package_Init - animation package proxy initialization
    --            Parameters
    --                Anim_Pack: address of proxy item 
    --                Package_Name: name of package        
    --                PackVtbl_Ptr: address of instrumentation table(for global variables 
    --                                and relations serialization)
    --------------------------------------------------------------------------------

    Procedure Package_Init (
          Anim_Pack    : Package_Acc_T; 
          Package_Name : String_Address;        
          PackVtbl_Ptr : System.Address ); 
    Pragma Import(C, Entity => Package_Init,External_Name =>"anim_pack_init");


    --------------------------------------------------------------------------------
    --            Class_Init - animation class proxy initialization
    --            Parameters
    --                Anim_Class: address of proxy item 
    --                Name: name of class
    --                Is_Singleton: TRUE,        if the class is singleton 
    --                Is_Implicit: 
    --                Package_Name: name of owner package        
    --                ClassVtbl_Ptr: address of instrumentation table(for attributes 
    --                                and relations serialization)
    --------------------------------------------------------------------------------
    Procedure Class_Init (
          Anim_Class    : Class_Acc_T;   
          Name          : String_Address;        
          Is_Singleton  : Boolean;       
          Is_Implicit   : Boolean;       
          Package_Name  : String_Address;        
          ClassVtbl_Ptr : System.Address ); 
    Pragma Import(C, Entity => Class_Init,External_Name =>"anim_class_init");

    --------------------------------------------------------------------------------
    -- destructors of animation proxy items        
    --------------------------------------------------------------------------------

    Procedure Package_Cleanup (
          Anim_Pack : Package_Acc_T ); 
    Procedure Class_Cleanup (
          Anim_Class : Class_Acc_T ); 

    --------------------------------------------------------------------------------
    --    CStr - convertor from Ada String to C NULL-terminated string
    --        Parameters
    --            AdaString: Ada string 
    --        Returns C string
    --------------------------------------------------------------------------------
    Function CStr (
          AdaString : String ) 
      Return String; 


    ------------------------- Reactive class interface ------------------------------

    Procedure Notify_Start_Behavior (
          This : RhpAnim.Reactive_Acc_T ); 
    Pragma Import(C, Entity => Notify_Start_Behavior,External_Name =>"anim_notify_start_behavior");

    Procedure Add_State (
          Anim_State : State_Acc_T; 
          StateName  : String_Address       ); 
    Pragma Import(C, Entity => Add_State,External_Name =>"anim_add_state");

    Procedure Notify_State_Entered (
          This       : UserClass_Acc_T; 
          Anim_Class : Class_Acc_T;     
          StateName  : String_Address ); 
    Pragma Import(C, Entity => Notify_State_Entered,External_Name =>"anim_notify_state_entered");

    Procedure Notify_State_Exited (
          This       : UserClass_Acc_T; 
          Anim_Class : Class_Acc_T;     
          StateName  : String_address    ); 
    Pragma Import(C, Entity => Notify_State_Exited,External_Name =>"anim_notify_state_exited");

    Procedure Notify_Transition_Started (
          This       : UserClass_Acc_T; 
          Anim_Class : Class_Acc_T;     
          Transition : String_address           ); 
    Pragma Import(C, Entity => Notify_Transition_Started,External_Name =>"anim_notify_transition_started");

    Procedure Notify_Transition_Terminated (
          This       : UserClass_Acc_T; 
          Anim_Class : Class_Acc_T;     
          Transition : String_address    ); 
    Pragma Import(C, Entity => Notify_Transition_Terminated,External_Name =>"anim_notify_transition_terminated");
    ------------------ Event animation services -------------------

    Procedure Event_Init (
          Event                     : Event_Acc_T;    
          Event_Name,                                 
          Event_Signature           : String_address;         
          Param_Number              : Integer;        
          Serialize_Function_Addr,                    
          Unserialize_Function_Addr : System.Address; 
          Package_Name              : String_address          ); 
    Pragma Import(C, Entity => Event_Init,External_Name =>"anim_event_init");

    Procedure Notify_Event_Constructor (
          Dest         : UserClass_Acc_T; 
          TheUserEvent : UserClass_Acc_T; 
          AnimEvent    : Event_Acc_T;     
          EventId      : Integer          ); 
    Pragma Import(C, Entity => Notify_Event_Constructor,External_Name =>"anim_notify_event_constructor");


    Procedure Notify_Gen_Event (
          This         :        UserClass_Acc_T; 
          TheUserEvent : In     Event_Acc_T;     
          AnimEvent    :        Event_Acc_T;     
          EventId      :        Integer          ); 
    Pragma Import(C, Entity => Notify_Gen_Event,External_Name =>"anim_notify_gen_event");


    Function Notify_Take_Event (
          This         :        UserClass_Acc_T; 
          TheUserEvent : In     Event_Acc_T;
          aomCallStackMark:     System.Address) 
      Return System.Address; 
    Pragma Import(C, Entity => Notify_Take_Event,External_Name =>"anim_notify_take_event");


    Procedure Notify_Event_Received (
          TheUserEvent : Event_Acc_T ); 
    Pragma Import(C, Entity => Notify_Event_Received,External_Name =>"anim_notify_event_received");

    Procedure Notify_Event_Consumed (
          TheUserEvent    : Event_Acc_T;   
          Call_Stack_Mark : System.Address ); 
    Pragma Import(C, Entity => Notify_Event_Consumed,External_Name =>"anim_notify_event_consumed");


    Function Unserialize_Event (
          Data           : System.Address; 
          AnimEvent      : Event_Acc_T;    
          Anim_Attr_List : System.Address  ) 
      Return Event_Acc_T; 
    Pragma Import(C, Entity => Unserialize_Event,External_Name =>"anim_unserialize_event");

    Procedure Notify_Event (
          User_Task : Task_Acc_T ); 
    Pragma Import(C, Entity => Notify_Event,External_Name =>"anim_notify_event");

    Procedure Notify_Idle (
          User_Task : Task_Acc_T ); 
    Pragma Import(C, Entity => Notify_Idle,External_Name =>"anim_notify_idle");

    Procedure Wakeup_Event_Queue (
          Signal_Ptr : RiA_Event_Flag_Ada83.RiA_Event_Flag_Ada83_Acc_T ); 
    Pragma Export(C, Entity => Wakeup_Event_Queue,External_Name =>"wakeup_event_queue");

    function Is_Stack_Growing_Down return boolean;
    Pragma Export(C, Entity => Is_Stack_Growing_Down,External_Name =>"is_stack_growing_down");

    Procedure Serialize_Attributes (
          C_List    : System.Address; 
          Attr_List : System.Address  ); 
    Pragma Import(C, Entity => Serialize_Attributes,External_Name =>"anim_serialize_attributes");

    ---------------------------------------------------------------
    -- Add_Relation add animated relation to the Rhapsody table
    --    Parameters:
    --        Anim_Relations - List of Relations (C framework internal)
    --        RelationName   - Role name         
    --        Is_Composite   TRUE, if global instance is defined        
    --        Is_Singular   TRUE, if relation is singular

    ---------------------------------------------------------------
    Procedure Add_Relation (
          Anim_Relations : Relations_Acc_T; 
          RelationName   : String_address;          
          Is_Composite   : Boolean;         
          Is_Singular    : Boolean); 
    Pragma Import(C, Entity => Add_Relation,External_Name =>"anim_add_relation");

    ---------------------------------------------------------------
    -- Add_Relation_Item adds animated relation item to the proxy relation
    --    Parameters:
    --        Anim_Relations - List of Relations (C framework internal)
    --        Relation_This  - Address of relation instance
    --        Anim_Class     animation proxy of owner class     
    ---------------------------------------------------------------
    Procedure Add_Relation_Item (
          Anim_Relations : Relations_Acc_T; 
          Relation_This  : UserClass_Acc_T; 
          Anim_Class     : Class_Acc_T    ); 
    Pragma Import(C, Entity => Add_Relation_Item,External_Name =>"anim_add_relation_item");

    ---------------------------------------------------------------
    -- Notify_Relation_Added initializes animation proxy relation item 
    --    Parameters:
    --        This                  - address of owner class instance
    --        Anim_Class            - animation proxy of owner class     
    --        Relation_Anim_Class   - animation proxy of related class     
    --        RelationName   - Role name         
    --        Relation_This  - Address of relation instance
    --        Is_Composite   TRUE, if global instance is defined        
    --        Is_Singular   TRUE, if relation is singular
    ---------------------------------------------------------------
    Procedure Notify_Relation_Added (
          This                : UserClass_Acc_T; 
          Anim_Class          : Class_Acc_T;     
          Relation_Anim_Class : Class_Acc_T;     
          RelationName        : String_Address;          
          Relation_This       : UserClass_Acc_T; 
          Is_Composite        : Boolean;         
          Is_Singular        : Boolean); 
    Pragma Import(C, Entity => Notify_Relation_Added,External_Name =>"anim_notify_relation_item_added");

    ---------------------------------------------------------------
    -- Notify_Relation_Cleared cleanups animation proxy relation 
    --    Parameters:
    --        This                  - address of owner class instance
    --        Anim_Class            - animation proxy of owner class     
    --        RelationName   - Role name         
    ---------------------------------------------------------------
    Procedure Notify_Relation_Cleared (
          This         : UserClass_Acc_T; 
          Anim_Class   : Class_Acc_T;     
          RelationName : String_Address           ); 
    Pragma Import(C, Entity => Notify_Relation_Cleared ,External_Name =>"anim_notify_relation_cleared");

    ---------------------------------------------------------------
    -- Notify_Relation_Removed deletes animation proxy relation item 
    --    Parameters:
    --        This                  - address of owner class instance
    --        Anim_Class            - animation proxy of owner class     
    --        Relation_Anim_Class   - animation proxy of related class     
    --        RelationName   - Role name         
    --        Relation_This  - Address of relation instance
    ---------------------------------------------------------------
    Procedure Notify_Relation_Removed (
          This                : UserClass_Acc_T; 
          Anim_Class          : Class_Acc_T;     
          Relation_Anim_Class : Class_Acc_T;     
          RelationName        : String_Address;          
          Relation_This       : UserClass_Acc_T  ); 
    Pragma Import(C, Entity => Notify_Relation_Removed,External_Name =>"anim_notify_relation_item_removed");

  task RiA_Connection_task is
    entry Start (this : in System.Address);
  end RiA_Connection_task;

  procedure Read_From_Socket(this : in System.Address);
  pragma Import (C,Read_From_Socket,"readFromSocket");
  
  procedure Connect_To_Socket(this : in System.Address);
  Pragma Export (C,Connect_To_Socket,"anim_connect_to_socket");

private

  -------------------------------------------------------------------------
  --	Enter_Exit an enter-exit object that calls Notify_End_Operation
  -------------------------------------------------------------------------
  type Enter_Exit is new CONTROLLED with record
    null;
  end record;

  procedure Initialize(this : in out Enter_Exit);
  procedure Finalize(this : in out Enter_Exit);
  
  -------------------------------------------------------------------------
  --	Notify_End_Operation - notifies Rhapsody about end of operation call
  -------------------------------------------------------------------------
  Procedure Notify_End_Operation;
  Pragma Import(C, Entity => Notify_End_Operation,External_Name =>"anim_notify_end_operation");

  -------------------------------------------------------------------------
  --	Enter_Exit an enter-exit object that calls Notify_End_Constructor
  -------------------------------------------------------------------------
  type Enter_Exit_Constructor is new CONTROLLED with record
    null;
  end record;

  procedure Initialize(this : in out Enter_Exit_Constructor);
  procedure Finalize(this : in out Enter_Exit_Constructor);
  
  -------------------------------------------------------------------------
  --	Notify_End_Constructor - notifies Rhapsody about finishing of 
  --		construction process
  -------------------------------------------------------------------------
  Procedure Notify_End_Constructor;
  Pragma Import(C, Entity => Notify_End_Constructor,External_Name =>"anim_notify_end_constructor");

  -------------------------------------------------------------------------
  --	Notify_Sched_Timeout - notifies Rhapsody about timeout scheduling 
  -------------------------------------------------------------------------
  Procedure Notify_Sched_Timeout (TheUserEvent : In     Event_Acc_T); 
  Pragma Import(C, Entity => Notify_Sched_Timeout,External_Name =>"anim_notify_sched_tm");
  
  -------------------------------------------------------------------------
  --        ada_timer_suspend - Rhapsody uspdates timer attributes 
  -------------------------------------------------------------------------
  procedure ada_timer_suspend;
  Pragma Export(C, Entity => ada_timer_suspend,External_Name =>"ada_timer_suspend");
 
  procedure ada_timer_resume;
  Pragma Export(C, Entity => ada_timer_resume,External_Name =>"ada_timer_resume");
  

End RhpAnim;

